#include "search_star.h"
#include <stdio.h>
#include <math.h>

enum
{
	VAR_TYPE_NORMAL = 0,
	VAR_TYPE_FLARE_STAR = 1,
	VAR_TYPE_CEPHEID = 2
};

struct RawMagnitude
{
	bool	bApparentVMag;
	double	ApparentVMag;
	bool	bVariableType;
	char	VariableType[4];
	bool	bVMagIncrease;
	double	VMagIncrease;
	bool	bVariablePeriod;
	double	VariablePeriod;
	bool	bBminusV;
	double	BminusV;
	bool	bUminusB;
	double	UminusB;
	bool	bRminusI;
	double	RminusI;
	bool	bVMagSource;
	char	VMagSource[11];
	DATE_STRUCT	LastUpdated;
};

static DATE_STRUCT sMostRecentUpdate;

static struct RawComponent
{
	int			ID;
	char		Name[11];
	bool		IsCloseBinary;
	bool		IsDiameterArcsec;
	DATE_STRUCT	LastUpdated;
	bool		bHasSpectrum, bHasMagnitude;
	RawSpectrum	spec;
	RawMagnitude mag;
	bool		bMass;
	double		Mass;
	bool		bDiameter;
	double		Diameter;	// in solar diameters if !IsDiameterArcsec
	bool		bMassSource;
	char		MassSource[11];
	bool		bDiameterSource;
	char		DiameterSource[11];
} sRawComponents[9];
static int sNumComponents = 0;

static struct RawPosition sSystemPosition, sComponentPosition[9];

static struct DerivedComponent
{
	double	AbsVMag;
	double	VLuminosity;		// in solar units
	double	ComfortZoneAUs;
	double	IncreasedAbsVMag;	// from variability
	double	IncreasedVLuminosity;
} sDerivedComponents[9];

static struct RawVelocity
{
	bool		bProperMotion, bPMAngle, bRadialVelocity;
	bool		bPMSource, bRVSource;
	bool		bRefined;	// if a component velocity, any new data?
	double		ProperMotion;
	double		PMAngle;
	double		RadialVelocity;
	char		PMSource[11];
	char		RVSource[11];
	DATE_STRUCT	LastUpdated;
} sSystemVelocity, sComponentVelocity[9];

static struct RawOrbit
{
	int			ID, PrimaryID, SecondaryID;
	bool		bPeriod, bSemimajorAxis, bEccentricity, bInclination;
	bool		bOmega, bPeriastronEpoch, bSource, bPeriodError;
	double		Period, SemimajorAxis, Eccentricity, Inclination;
	double		Omega, PeriastronEpoch;
	bool		IsAxisJustGap, IsAxisAU;
	char		Source[51];
	double		PeriodError;
	DATE_STRUCT	LastUpdated;
	bool		bHasSpectrum, bHasMagnitude;
	RawSpectrum	spec;
	RawMagnitude mag;
} sRawOrbits[39];
static int sNumOrbits = 0;

static struct RawPlanet
{
	int			ID;
	char		Name[51];
	DATE_STRUCT	LastUpdated;
	bool		bMass;
	double		Mass;
	bool		bMassError;
	double		MassError;
	bool		IsMassAMinimum;
	bool		bMassSource;
	char		MassSource[51];
	bool		bRadius;
	double		Radius;
	bool		bRadiusError;
	double		RadiusError;
	bool		bRadiusSource;
	char		RadiusSource[51];
} sRawPlanets[19];
static int sNumPlanets = 0;


// I can't believe they don't already have this in the ODBC API:
static int CompareDates (const DATE_STRUCT *a, const DATE_STRUCT *b)
{
	if (a->year > b->year)
		return	1;
	if (a->year < b->year)
		return	-1;
	if (a->month > b->month)
		return	1;
	if (a->month < b->month)
		return	-1;
	if (a->day > b->day)
		return	1;
	if (a->day < b->day)
		return	-1;
	return	0;
}

static bool RDDPAR2UVW (const struct RawPosition *pos,
	const struct RawVelocity *vel,
	double &u, double &v, double &w, bool bUseGalacticCoords)
{
	double	pa_rad, pm_kps;
	double	theta, phi;
	int		dec_sign;
	double	cu, cv, cw;		// u, v, and w in celestial coords

	if (!pos->bDistance)
		return	false;

	if (pos->Distance == 0.0)
	{
		u = v = w = 0.0;	// special case for our sun
		return	true;
	}

	if (!pos->bRA  ||  !pos->bDec  ||  !vel->bProperMotion  ||
		!vel->bPMAngle	||  !vel->bRadialVelocity)
			return	false;

	theta = (pos->RA_hr + pos->RA_min/60.0 + pos->RA_sec/3600.0)
		* PI / 12.0;

	if (pos->Dec_deg < 0)
		dec_sign = -1;
	else
		dec_sign = 1;
	phi = (90.0 - (pos->Dec_deg + pos->Dec_arcmin*dec_sign/60.0 +
			pos->Dec_arcsec*dec_sign/3600.0)) * PI / 180.0;

	pa_rad = vel->PMAngle * PI / 180.0;	// convert to radians

	pm_kps = 299792.458 * pos->Distance *
		tan(vel->ProperMotion / 3600.0 * PI / 180.0);

	cu = vel->RadialVelocity * cos(theta) * cos(PI/2.0-phi) -
		pm_kps * sin(theta) * sin(pa_rad) -
		pm_kps * cos(theta) * sin(PI/2.0-phi) * cos(pa_rad);
	cv = vel->RadialVelocity * sin(theta) * cos(PI/2.0-phi) +
		pm_kps * cos(theta) * sin(pa_rad) -
		pm_kps * sin(theta) * sin(PI/2.0-phi)*cos(pa_rad);
	cw = vel->RadialVelocity * sin(PI/2.0-phi) +
		pm_kps * cos(PI/2.0-phi) * cos(pa_rad);

	if (bUseGalacticCoords)
	{
		u = -0.0672*cu - 0.8727*cv - 0.4835*cw;
		v =  0.4927*cu - 0.4504*cv + 0.7445*cw;
		w = -0.8676*cu - 0.1884*cv + 0.4602*cw;
	}
	else
	{
		u = cu;
		v = cv;
		w = cw;
	}
	return	true;
}

static bool FetchRawMagnitude (HSTMT hstmt, long id, RawMagnitude &mag)
{
	char	buffer[512];
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT ApparentVMag, VMagIncrease, "
		"VariableType, VariablePeriod, "
		"BminusV, UminusB, RminusI, "
		"VMagSource, LastUpdated FROM Magnitudes "
		"WHERE OwnerID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		SDWORD	cbApparentVMag, cbVMagIncrease, cbVariableType;
		SDWORD	cbVariablePeriod, cbBminusV, cbUminusB, cbRminusI;
		SDWORD	cbVMagSource, cbLastUpdated;

		SQLBindCol (hstmt, 1, SQL_C_DOUBLE, &mag.ApparentVMag, 0, &cbApparentVMag);
		SQLBindCol (hstmt, 2, SQL_C_DOUBLE, &mag.VMagIncrease, 0, &cbVMagIncrease);
		SQLBindCol (hstmt, 3, SQL_C_CHAR, mag.VariableType, 4, &cbVariableType);
		SQLBindCol (hstmt, 4, SQL_C_DOUBLE, &mag.VariablePeriod, 0, &cbVariablePeriod);
		SQLBindCol (hstmt, 5, SQL_C_DOUBLE, &mag.BminusV, 0, &cbBminusV);
		SQLBindCol (hstmt, 6, SQL_C_DOUBLE, &mag.UminusB, 0, &cbUminusB);
		SQLBindCol (hstmt, 7, SQL_C_DOUBLE, &mag.RminusI, 0, &cbRminusI);
		SQLBindCol (hstmt, 8, SQL_C_CHAR, mag.VMagSource, 11, &cbVMagSource);
		SQLBindCol (hstmt, 9, SQL_C_DATE, &mag.LastUpdated, 0, &cbLastUpdated);
		retcode = SQLFetch (hstmt);
		if (retcode == SQL_SUCCESS  ||  retcode == SQL_SUCCESS_WITH_INFO)
		{
			mag.bApparentVMag = cbApparentVMag != SQL_NULL_DATA;
			mag.bVMagIncrease = cbVMagIncrease != SQL_NULL_DATA;
			mag.bVariableType = cbVariableType != SQL_NULL_DATA;
			mag.bVariablePeriod = cbVariablePeriod != SQL_NULL_DATA;
			mag.bBminusV = cbBminusV != SQL_NULL_DATA;
			mag.bUminusB = cbUminusB != SQL_NULL_DATA;
			mag.bRminusI = cbRminusI != SQL_NULL_DATA;
			mag.bVMagSource = cbVMagSource != SQL_NULL_DATA;
			if (CompareDates (&mag.LastUpdated, &sMostRecentUpdate) > 0)
				sMostRecentUpdate = mag.LastUpdated;
			return	true;
		}
	}
	return	false;
}

static void FetchRawComponents (HDBC hdbc, HSTMT hstmt, long id)
{
	char	buffer[512];
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT ID, Name, IsCloseBinary, Mass, "
		"Diameter, LastUpdated, MassSource, "
		"DiameterSource, IsDiameterArcsec FROM Components "
		"WHERE ID > %ld AND ID <= %ld ORDER BY ID", id, id+9);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		struct RawComponent comp;
		SDWORD	cbID, cbName, cbIsCloseBinary, cbLastUpdated;
		SDWORD	cbMass, cbDiameter;
		SDWORD	cbMassSource, cbDiameterSource, cbIsDiameterArcsec;

		SQLBindCol (hstmt, 1, SQL_C_LONG, &comp.ID, 0, &cbID);
		SQLBindCol (hstmt, 2, SQL_C_CHAR, comp.Name, 11, &cbName);
		SQLBindCol (hstmt, 3, SQL_C_BIT, &comp.IsCloseBinary, 0, &cbIsCloseBinary);
		SQLBindCol (hstmt, 4, SQL_C_DOUBLE, &comp.Mass, 0, &cbMass);
		SQLBindCol (hstmt, 5, SQL_C_DOUBLE, &comp.Diameter, 0, &cbDiameter);
		SQLBindCol (hstmt, 6, SQL_C_DATE, &comp.LastUpdated, 0, &cbLastUpdated);
		SQLBindCol (hstmt, 7, SQL_C_CHAR, comp.MassSource, 11, &cbMassSource);
		SQLBindCol (hstmt, 8, SQL_C_CHAR, comp.DiameterSource, 11, &cbDiameterSource);
		SQLBindCol (hstmt, 9, SQL_C_BIT, &comp.IsDiameterArcsec, 0, &cbIsDiameterArcsec);
		while (true)
		{
			retcode = SQLFetch(hstmt);
			if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
				break;
			comp.bMass = cbMass != SQL_NULL_DATA;
			comp.bDiameter = cbDiameter != SQL_NULL_DATA;
			comp.bMassSource = cbMassSource != SQL_NULL_DATA;
			comp.bDiameterSource = cbDiameterSource != SQL_NULL_DATA;

			comp.bHasSpectrum = false;
			comp.spec.bSpectralClass = false;
			comp.spec.bLuminosityClass = false;
			comp.spec.bSpectrumSource = false;
			comp.bHasMagnitude = false;
			comp.mag.bApparentVMag = false;
			comp.mag.bVMagIncrease = false;
			comp.mag.bVariableType = false;
			comp.mag.bVariablePeriod = false;
			comp.mag.bBminusV = false;
			comp.mag.bUminusB = false;
			comp.mag.bRminusI = false;
			comp.mag.bVMagSource = false;
			HSTMT hStmt2;
			retcode = SQLAllocStmt(hdbc, &hStmt2);
			if (retcode == SQL_SUCCESS)
			{
				if (FetchRawSpectrum (hStmt2, comp.ID, comp.spec))
				{
					comp.bHasSpectrum = true;
					if (CompareDates (&comp.spec.LastUpdated, &comp.LastUpdated) > 0)
						comp.LastUpdated = comp.spec.LastUpdated;
				}

				if (FetchRawMagnitude (hStmt2, comp.ID, comp.mag))
				{
					comp.bHasMagnitude = true;
					if (CompareDates (&comp.mag.LastUpdated, &comp.LastUpdated) > 0)
						comp.LastUpdated = comp.mag.LastUpdated;
				}

				SQLFreeStmt(hStmt2, SQL_DROP);
			}
			if (CompareDates (&comp.LastUpdated, &sMostRecentUpdate) > 0)
				sMostRecentUpdate = comp.LastUpdated;

			sRawComponents [sNumComponents] = comp;
			++ sNumComponents;
		}
	}
}

static bool FetchSystemAndComponentRawPositions (HSTMT hstmt, long id)
{
	if (!FetchRawPosition (hstmt, id, &sSystemPosition))
		return	false;
	if (CompareDates (&sSystemPosition.LastUpdated, &sMostRecentUpdate) > 0)
		sMostRecentUpdate = sSystemPosition.LastUpdated;
	for (int index = 0; index < sNumComponents; ++index)
	{
		sComponentPosition[index].bRefinedRA = false;
		sComponentPosition[index].bRefinedDec = false;
		sComponentPosition[index].bRefinedDist = false;
		if (FetchRawPosition (hstmt, sRawComponents[index].ID, &sComponentPosition[index]))
		{
			if (sComponentPosition[index].bRA)
				sComponentPosition[index].bRefinedRA = true;
			else if (sSystemPosition.bRA)
			{
				sComponentPosition[index].RA_hr = sSystemPosition.RA_hr;
				sComponentPosition[index].RA_min = sSystemPosition.RA_min;
				sComponentPosition[index].RA_sec = sSystemPosition.RA_sec;
				sComponentPosition[index].bRA = true;
			}

			if (sComponentPosition[index].bDec)
				sComponentPosition[index].bRefinedDec = true;
			else if (sSystemPosition.bDec)
			{
				sComponentPosition[index].Dec_deg = sSystemPosition.Dec_deg;
				sComponentPosition[index].Dec_arcmin = sSystemPosition.Dec_arcmin;
				sComponentPosition[index].Dec_arcsec = sSystemPosition.Dec_arcsec;
				sComponentPosition[index].bDec = true;
			}

			if (sComponentPosition[index].bDistance)
				sComponentPosition[index].bRefinedDist = true;
			else if (sSystemPosition.bDistance)
			{
				sComponentPosition[index].Distance = sSystemPosition.Distance;
				sComponentPosition[index].bDistance = true;
				sComponentPosition[index].bDistanceError = sSystemPosition.bDistanceError;
				if (sComponentPosition[index].bDistanceError)
					sComponentPosition[index].DistanceError = sSystemPosition.DistanceError;
				sComponentPosition[index].bDistanceSource = sSystemPosition.bDistanceSource;
				if (sComponentPosition[index].bDistanceSource)
					strcpy (sComponentPosition[index].DistanceSource, sSystemPosition.DistanceSource);
			}

			if (CompareDates (&sSystemPosition.LastUpdated,
					&sComponentPosition[index].LastUpdated) > 0)
				sComponentPosition[index].LastUpdated = sSystemPosition.LastUpdated;
			else if (CompareDates (&sComponentPosition[index].LastUpdated,
					&sMostRecentUpdate) > 0)
				sMostRecentUpdate = sComponentPosition[index].LastUpdated;
		}
		else
			sComponentPosition[index] = sSystemPosition;
	}
	return	true;
}

static void ComputeDerivedComponents ()
{
	for (int index = 0; index < sNumComponents; ++index)
	{
		if (!sRawComponents[index].bHasMagnitude  ||
			!sRawComponents[index].mag.bApparentVMag  ||
			!sComponentPosition[index].bDistance)
		{
			sDerivedComponents[index].AbsVMag = -9999.0;
			sDerivedComponents[index].VLuminosity = -1.0;
			sDerivedComponents[index].ComfortZoneAUs = -1.0;
			sDerivedComponents[index].IncreasedAbsVMag = -9999.0;
			sDerivedComponents[index].IncreasedVLuminosity = -1.0;
			continue;
		}

		if (!sComponentPosition[index].Distance)			
			sDerivedComponents[index].AbsVMag = 4.85;	// special case for our own sun
		else
			sDerivedComponents[index].AbsVMag = sRawComponents[index].mag.ApparentVMag
				+ 5.0 - 5.0 * log10 (sComponentPosition[index].Distance / LY_PER_PARSEC);

		sDerivedComponents[index].IncreasedAbsVMag = sDerivedComponents[index].AbsVMag;
		if (sRawComponents[index].mag.bVMagIncrease)
			sDerivedComponents[index].IncreasedAbsVMag -= sRawComponents[index].mag.VMagIncrease;

		sDerivedComponents[index].VLuminosity =
			pow (2.512, 4.85 - sDerivedComponents[index].AbsVMag);
		sDerivedComponents[index].IncreasedVLuminosity =
			pow (2.512, 4.85 - sDerivedComponents[index].IncreasedAbsVMag);

		sDerivedComponents[index].ComfortZoneAUs =
			sqrt (sDerivedComponents[index].VLuminosity);
	}
}

static bool FetchRawVelocity (HSTMT hstmt, long id, struct RawVelocity *vel)
{
	char	buffer[512];
	RETCODE	retcode;
	SDWORD	cbProperMotion, cbPMAngle, cbRadialVelocity;
	SDWORD	cbPMSource, cbRVSource, cbLastUpdated;

	memset (vel, 0, sizeof *vel);	// initialize everything to false

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT ProperMotion, PMAngle, RadialVelocity, "
		"PMSource, RVSource, LastUpdated "
		"FROM Velocities WHERE OwnerID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode != SQL_SUCCESS)
		return	false;

	SQLBindCol(hstmt, 1, SQL_C_DOUBLE, &vel->ProperMotion, 0, &cbProperMotion);
	SQLBindCol(hstmt, 2, SQL_C_DOUBLE, &vel->PMAngle, 0, &cbPMAngle);
	SQLBindCol(hstmt, 3, SQL_C_DOUBLE, &vel->RadialVelocity, 0, &cbRadialVelocity);
	SQLBindCol(hstmt, 4, SQL_C_CHAR, &vel->PMSource, 11, &cbPMSource);
	SQLBindCol(hstmt, 5, SQL_C_CHAR, &vel->RVSource, 11, &cbRVSource);
	SQLBindCol(hstmt, 6, SQL_C_DATE, &vel->LastUpdated, 0, &cbLastUpdated);
	retcode = SQLFetch(hstmt);
	if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
		return	false;
	vel->bProperMotion = cbProperMotion != SQL_NULL_DATA;
	vel->bPMAngle = cbPMAngle != SQL_NULL_DATA;
	vel->bRadialVelocity = cbRadialVelocity != SQL_NULL_DATA;
	vel->bPMSource = cbPMSource != SQL_NULL_DATA;
	vel->bRVSource = cbRVSource != SQL_NULL_DATA;
	if (CompareDates (&vel->LastUpdated, &sMostRecentUpdate) > 0)
		sMostRecentUpdate = vel->LastUpdated;
	return	true;
}

static bool FetchSystemAndComponentRawVelocities (HSTMT hstmt, long id)
{
	FetchRawVelocity (hstmt, id, &sSystemVelocity);
	for (int index = 0; index < sNumComponents; ++index)
	{
		if (FetchRawVelocity (hstmt, sRawComponents[index].ID, &sComponentVelocity[index]))
		{
			sComponentVelocity[index].bRefined = true;
			if (!sComponentVelocity[index].bProperMotion  &&  sSystemVelocity.bProperMotion)
			{
				sComponentVelocity[index].ProperMotion = sSystemVelocity.ProperMotion;
				sComponentVelocity[index].bProperMotion = true;
				sComponentVelocity[index].bPMSource = sSystemVelocity.bPMSource;
				if (sSystemVelocity.bPMSource)
					strcpy (sComponentVelocity[index].PMSource, sSystemVelocity.PMSource);
			}
			if (!sComponentVelocity[index].bPMAngle  &&  sSystemVelocity.bPMAngle)
			{
				sComponentVelocity[index].PMAngle = sSystemVelocity.PMAngle;
				sComponentVelocity[index].bPMAngle = true;
			}
			if (!sComponentVelocity[index].bRadialVelocity  &&
				sSystemVelocity.bRadialVelocity)
			{
				sComponentVelocity[index].RadialVelocity = sSystemVelocity.RadialVelocity;
				sComponentVelocity[index].bRadialVelocity = true;
				sComponentVelocity[index].bRVSource = sSystemVelocity.bRVSource;
				if (sSystemVelocity.bRVSource)
					strcpy (sComponentVelocity[index].RVSource, sSystemVelocity.RVSource);
			}
			if (CompareDates (&sSystemPosition.LastUpdated,
					&sComponentVelocity[index].LastUpdated) > 0)
				sComponentVelocity[index].LastUpdated = sSystemVelocity.LastUpdated;
		}
		else
		{
			sComponentVelocity[index] = sSystemVelocity;
			sComponentVelocity[index].bRefined = false;
		}
	}
	return	true;
}

static void CatAbbrev2LongNamePlusAbbrev (const char *abbrev, char *buf)
{
	const char *longName = CatAbbrev2LongName (abbrev);
	if (longName == abbrev)
		strcpy (buf, abbrev);
	else
		sprintf (buf, "%s (%s)", longName, abbrev);
}

static void DisplayRADecDist (const struct RawPosition *pos, bool isComponent)
{
	if (!isComponent  ||  pos->bRefinedRA  ||  pos->bRefinedDec)
	{
		if (pos->bRA  &&  pos->bDec)
			printf ("<BR><B>Right Ascension and Declination:</B>");
		else if (pos->bRA)
			printf ("<BR><B>Right Ascension:</B>");
		else if (pos->bDec)
			printf ("<BR><B>Declination:</B>");
		if (pos->bRA)
			printf (" %dh%dm%gs", (int) pos->RA_hr,
				(int) pos->RA_min, (double) pos->RA_sec);
		if (pos->bDec)
		{
			if (pos->bRA)
				printf (",");
			printf (" %+d&deg;%d'%g\"", (int) pos->Dec_deg,
				(int) pos->Dec_arcmin, (double) pos->Dec_arcsec);
		}
		if (pos->bRA  ||  pos->bDec)
			printf (" (epoch %.01f)\n", pos->Epoch);
	}
	if (!isComponent  ||  pos->bRefinedDist)
	{
		if (pos->bDistance)
		{
			printf ("<BR><B>Distance from Sol:</B> ");
			if (pos->bDistanceError)
			{
				printf ("%s light-years",
					FormatWithSignificantDigits
						(pos->Distance, pos->DistanceError));
				printf (" (%s parsecs)\n",
					FormatWithSignificantDigits
						(pos->Distance / LY_PER_PARSEC, pos->DistanceError));
			}
			else
				printf ("%g light-years (%g parsecs)\n",
					pos->Distance, pos->Distance / LY_PER_PARSEC);
		}
		if (pos->bDistanceError)
			printf ("<BR><B>Standard error in distance:</B> %g%%\n",
				pos->DistanceError * 100.0);
		if (pos->bDistanceSource)
			printf ("<BR><B>Source for distance:</B> %s\n",
				CatAbbrev2LongName (pos->DistanceSource));
	}
}

static void DisplayPMPARV (const struct RawVelocity *vel)
{
	if (vel->bProperMotion)
	{
		printf ("<BR><B>Proper motion:</B> %g arcsec/yr", vel->ProperMotion);
		if (vel->bPMAngle)
			printf (" (%.01f&deg; from north)", vel->PMAngle);
		printf ("\n");
	}
	if (vel->bRadialVelocity)
		printf ("<BR><B>Radial Velocity:</B> %g km/sec\n", vel->RadialVelocity);
	if (vel->bPMSource  &&  vel->bRVSource  &&
		!strcmp (vel->PMSource, vel->RVSource))
			printf ("<BR><B>Source for proper motion and radial velocity:</B> %s\n",
				CatAbbrev2LongName (vel->PMSource));
	else
	{
		if (vel->bPMSource)
			printf ("<BR><B>Source for proper motion:</B> %s\n",
				CatAbbrev2LongName (vel->PMSource));
		if (vel->bRVSource)
			printf ("<BR><B>Source for radial velocity:</B> %s\n",
				CatAbbrev2LongName (vel->RVSource));
	}
}

static void FetchAndDisplaySystemNames (HSTMT hstmt, long id)
{
	char	buffer[512];
	RETCODE	retcode;

	if (FetchMainSystemName (hstmt, id, buffer))
		printf ("<CENTER><H1>%s</H1></CENTER>\n", buffer);

	puts ("<FORM ACTION=\"/Scripts/find_neighbors.exe\" METHOD=\"GET\">");
	printf ("<INPUT TYPE=\"Hidden\" NAME=\"ID\" VALUE=\"%d\">\n", id);
	puts ("<INPUT TYPE=\"Submit\" VALUE=\"Find all stars within\">");
	puts ("<INPUT TYPE=\"text\" NAME=\"ly\" VALUE=\"15\" SIZE=3> light-years");
	puts ("</FORM>");

	SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT Name FROM ProperNames "
		"WHERE OwnerID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		SDWORD	cbName;
		char	name[51];
		bool	printedOne = false;

		SQLBindCol(hstmt, 1, SQL_C_CHAR, name, 51, &cbName);
		while (true)
		{
			retcode = SQLFetch(hstmt);
			if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
				break;
			if (printedOne)
				printf (", ");
			else
			{
				printf ("<B>Proper names:</B> ");
				printedOne = true;
			}
			printf ("%s", name);
		}
		if (printedOne)
			printf ("\n");
	}

	SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT CatAbbrev, Value FROM CatalogNumbers "
		"WHERE OwnerID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		char	catAbbrev[11], value[51];
		SDWORD	cbCatAbbrev, cbValue;
		bool	printedOne = false;

		SQLBindCol(hstmt, 1, SQL_C_CHAR, catAbbrev, 11, &cbCatAbbrev);
		SQLBindCol(hstmt, 2, SQL_C_CHAR, value, 51, &cbValue);
		while (true)
		{
			retcode = SQLFetch(hstmt);
			if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
				break;
			if (!printedOne)
			{
				printf ("<BR><B>Catalog numbers:</B>\n<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				printedOne = true;
			}
			else
				printf (", ");
			CatAbbrev2LongNamePlusAbbrev (catAbbrev, buffer);
			printf ("%s %s", buffer, value);
		}
		if (printedOne)
			printf (/* "<BR>\n" */ "\n");
	}
}

static void FetchAndDisplaySystemData (HSTMT hstmt, long id)
{
	char	buffer[512];
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT Age, AgeError, AgeSource, AgeComment, "
		"Metallicity, MetallicityError, MetallicitySource, MetallicityComment, "
		"ArityComment, PointsOfInterest, LastUpdated FROM Systems WHERE ID = %ld", id);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		double	age, ageError, metallicity, metallicityError;
		char	ageSource[51], metallicitySource[51];
		char	ageComment[512], metallicityComment[512], arityComment[512];
		char	pointsOfInterest[4096];
		DATE_STRUCT	lastUpdated;
		SDWORD	cbAge, cbAgeError, cbAgeSource, cbAgeComment;
		SDWORD	cbMetallicity, cbMetallicityError, cbMetallicitySource, cbMetallicityComment;
		SDWORD	cbArityComment, cbPointsOfInterest, cbLastUpdated;

		SQLBindCol(hstmt, 1, SQL_C_DOUBLE, &age, 0, &cbAge);
		SQLBindCol(hstmt, 2, SQL_C_DOUBLE, &ageError, 0, &cbAgeError);
		SQLBindCol(hstmt, 3, SQL_C_CHAR, ageSource, 51, &cbAgeSource);
		SQLBindCol(hstmt, 4, SQL_C_CHAR, ageComment, 512, &cbAgeComment);
		SQLBindCol(hstmt, 5, SQL_C_DOUBLE, &metallicity, 0, &cbMetallicity);
		SQLBindCol(hstmt, 6, SQL_C_DOUBLE, &metallicityError, 0, &cbMetallicityError);
		SQLBindCol(hstmt, 7, SQL_C_CHAR, metallicitySource, 51, &cbMetallicitySource);
		SQLBindCol(hstmt, 8, SQL_C_CHAR, metallicityComment, 512, &cbMetallicityComment);
		SQLBindCol(hstmt, 9, SQL_C_CHAR, arityComment, 512, &cbArityComment);
		SQLBindCol(hstmt, 10, SQL_C_CHAR, pointsOfInterest, 4096, &cbPointsOfInterest);
		SQLBindCol(hstmt, 11, SQL_C_DATE, &lastUpdated, 0, &cbLastUpdated);
		retcode = SQLFetch(hstmt);
		if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
			return;
		if (CompareDates (&lastUpdated, &sMostRecentUpdate) > 0)
			sMostRecentUpdate = lastUpdated;
		if (cbAge != SQL_NULL_DATA  ||  cbAgeComment != SQL_NULL_DATA)
		{
			printf ("<BR><B>Age:</B> ");
			if (cbAge != SQL_NULL_DATA)
			{
				printf ("%d million years", int (age));
				if (cbAgeComment != SQL_NULL_DATA)
					printf (" &mdash; ");
			}
			if (cbAgeComment != SQL_NULL_DATA)
				printf ("%s", ageComment);
			printf ("\n");
		}
		if (cbAgeError != SQL_NULL_DATA)
			printf ("<BR><B>Standard error in age:</B> %d%%\n", int (ageError * 100.0));
		if (cbAgeSource != SQL_NULL_DATA)
			printf ("<BR><B>Source for age:</B> %s\n", CatAbbrev2LongName (ageSource));
		if (cbMetallicity != SQL_NULL_DATA  ||  cbMetallicityComment != SQL_NULL_DATA)
		{
			printf ("<BR><B>Heavy element abundance:</B> ");
			if (cbMetallicity != SQL_NULL_DATA)
				printf ("%d%% of Sol", int (metallicity * 100.0 + 0.5));
			if (cbMetallicityComment != SQL_NULL_DATA)
				printf ("%s", metallicityComment);
			printf ("\n");
		}
		if (cbMetallicityError != SQL_NULL_DATA)
			printf ("<BR><B>Standard error in heavy element abundance:</B> %d%%\n", int (metallicityError * 100.0));
		if (cbMetallicitySource != SQL_NULL_DATA)
			printf ("<BR><B>Source for heavy element abundance:</B> %s\n",
				CatAbbrev2LongName (metallicitySource));

		printf ("<BR><B>Arity:</B> ");
		if (cbArityComment != SQL_NULL_DATA)
			printf ("%s", arityComment);
		else switch (sNumComponents)
		{
			case 1:
				printf ("singular");
				break;
			case 2:
				printf ("binary");
				break;
			case 3:
				printf ("trinary");
				break;
			case 4:
				printf ("quarternary (!)");
				break;
			case 5:
				printf ("quinary (!)");
				break;
			case 6:
				printf ("hexary (!!)");
				break;
			case 7:
				printf ("heptary (!!)");
				break;
			case 8:
				printf ("octary (!!!)");
				break;
			case 9:
				printf ("nonary (!!!)");
				break;
		}
		printf ("\n");
		if (cbPointsOfInterest != SQL_NULL_DATA)
			printf ("<BR><B>Points of interest:</B>\n<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
				"%s\n<BR>\n", pointsOfInterest);
	}
}

static void Display3DistanceDependentValues
	(double x, double y, double z, const struct RawPosition *pos)
{
	if (!pos->bDistanceError)
		printf ("%g, %g, %g", x, y, z);
	else
	{
		printf ("%s, ", FormatWithSignificantDigits (x, pos->DistanceError));
		printf ("%s, ", FormatWithSignificantDigits (y, pos->DistanceError));
		printf ("%s", FormatWithSignificantDigits (z, pos->DistanceError));
	}
}

static void DisplayCoords (const struct RawPosition *pos)
{
	double	x, y, z;

	if (RDD2XYZ (pos, x, y, z, false))
	{
		printf ("<BR><B>Celestial (X,Y,Z) coordinates in ly:</B> ");
		Display3DistanceDependentValues (x, y, z, pos);
		printf ("\n");
		if (RDD2XYZ (pos, x, y, z, true))
		{
			printf ("<BR><B>Galactic (X,Y,Z) coordinates in ly:</B> ");
			Display3DistanceDependentValues (x, y, z, pos);
			printf ("\n");
		}
	}
}

static void DisplaySystemVelocity ()
{
	double	u, v, w;

	DisplayPMPARV (&sSystemVelocity);
	if (RDDPAR2UVW (&sSystemPosition, &sSystemVelocity, u, v, w, true))
	{
		printf ("<BR><B>Galactic (U,V,W) velocity components in km/s:</B> ");
		Display3DistanceDependentValues (u, v, w, &sSystemPosition);
		printf ("\n");
	}
}

static void FetchRawOrbits (HDBC hdbc, HSTMT hstmt, long id)
{
	char	buffer[512];
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT ID, PrimaryID, SecondaryID, Period, "
		"SemimajorAxis, Eccentricity, Inclination, Omega, PeriastronEpoch, "
		"IsAxisJustGap, Source, PeriodError, LastUpdated, IsAxisAU "
		"FROM Orbits WHERE ID >= %ld AND ID <= %ld ORDER BY ID", id+11, id+49);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		struct RawOrbit	orb;
		SDWORD	cbID, cbPrimaryID, cbSecondaryID, cbPeriod, cbSemimajorAxis;
		SDWORD	cbEccentricity, cbInclination, cbOmega, cbPeriastronEpoch;
		SDWORD	cbIsAxisJustGap, cbSource, cbPeriodError, cbLastUpdated;
		SDWORD	cbIsAxisAU;

		SQLBindCol(hstmt, 1, SQL_C_LONG, &orb.ID, 0, &cbID);
		SQLBindCol(hstmt, 2, SQL_C_LONG, &orb.PrimaryID, 0, &cbPrimaryID);
		SQLBindCol(hstmt, 3, SQL_C_LONG, &orb.SecondaryID, 0, &cbSecondaryID);
		SQLBindCol(hstmt, 4, SQL_C_DOUBLE, &orb.Period, 0, &cbPeriod);
		SQLBindCol(hstmt, 5, SQL_C_DOUBLE, &orb.SemimajorAxis, 0, &cbSemimajorAxis);
		SQLBindCol(hstmt, 6, SQL_C_DOUBLE, &orb.Eccentricity, 0, &cbEccentricity);
		SQLBindCol(hstmt, 7, SQL_C_DOUBLE, &orb.Inclination, 0, &cbInclination);
		SQLBindCol(hstmt, 8, SQL_C_DOUBLE, &orb.Omega, 0, &cbOmega);
		SQLBindCol(hstmt, 9, SQL_C_DOUBLE, &orb.PeriastronEpoch, 0, &cbPeriastronEpoch);
		SQLBindCol(hstmt, 10, SQL_C_BIT, &orb.IsAxisJustGap, 0, &cbIsAxisJustGap);
		SQLBindCol(hstmt, 11, SQL_C_CHAR, &orb.Source, 51, &cbSource);
		SQLBindCol(hstmt, 12, SQL_C_DOUBLE, &orb.PeriodError, 0, &cbPeriodError);
		SQLBindCol(hstmt, 13, SQL_C_DATE, &orb.LastUpdated, 0, &cbLastUpdated);
		SQLBindCol(hstmt, 14, SQL_C_BIT, &orb.IsAxisAU, 0, &cbIsAxisAU);
		while (true)
		{
			retcode = SQLFetch(hstmt);
			if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
				break;
			orb.bPeriod = cbPeriod != SQL_NULL_DATA;
			orb.bSemimajorAxis = cbSemimajorAxis != SQL_NULL_DATA;
			orb.bEccentricity = cbEccentricity != SQL_NULL_DATA;
			orb.bInclination = cbInclination != SQL_NULL_DATA;
			orb.bOmega = cbOmega != SQL_NULL_DATA;
			orb.bPeriastronEpoch = cbPeriastronEpoch != SQL_NULL_DATA;
			orb.bSource = cbSource != SQL_NULL_DATA;
			orb.bPeriodError = cbPeriodError != SQL_NULL_DATA;
			orb.bHasSpectrum = orb.bHasMagnitude = false;
			HSTMT hStmt2;
			retcode = SQLAllocStmt(hdbc, &hStmt2);
			if (retcode == SQL_SUCCESS)
			{
				if (FetchRawSpectrum (hStmt2, orb.ID, orb.spec))
				{
					orb.bHasSpectrum = true;
					if (CompareDates (&orb.spec.LastUpdated, &orb.LastUpdated) > 0)
						orb.LastUpdated = orb.spec.LastUpdated;
				}

				if (FetchRawMagnitude (hStmt2, orb.ID, orb.mag))
				{
					orb.bHasMagnitude = true;
					if (CompareDates (&orb.mag.LastUpdated, &orb.LastUpdated) > 0)
						orb.LastUpdated = orb.mag.LastUpdated;
				}

				SQLFreeStmt(hStmt2, SQL_DROP);
			}
			if (CompareDates (&orb.LastUpdated, &sMostRecentUpdate) > 0)
				sMostRecentUpdate = orb.LastUpdated;

			sRawOrbits [sNumOrbits] = orb;
			++ sNumOrbits;
		}
	}
}

static void FetchRawPlanets (HDBC hdbc, HSTMT hstmt, long id)
{
	char	buffer[512];
	RETCODE	retcode;

	retcode = SQLFreeStmt (hstmt, SQL_CLOSE);
	sprintf (buffer, "SELECT ID, Name, LastUpdated, "
		"Mass, MassError, IsMassAMinimum, MassSource, "
		"Radius, RadiusError, RadiusSource "
		"FROM Planets WHERE ID >= %ld AND ID <= %ld ORDER BY ID", id+51, id+69);
	retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
	if (retcode == SQL_SUCCESS)
	{
		struct RawPlanet	planet;
		SDWORD	cbID, cbName, cbLastUpdated;
		SDWORD	cbMass, cbMassError, cbIsMassAMinimum, cbMassSource;
		SDWORD	cbRadius, cbRadiusError, cbRadiusSource;

		SQLBindCol(hstmt, 1, SQL_C_LONG, &planet.ID, 0, &cbID);
		SQLBindCol(hstmt, 2, SQL_C_CHAR, &planet.Name, 51, &cbName);
		SQLBindCol(hstmt, 3, SQL_C_DATE, &planet.LastUpdated, 0, &cbLastUpdated);
		SQLBindCol(hstmt, 4, SQL_C_DOUBLE, &planet.Mass, 0, &cbMass);
		SQLBindCol(hstmt, 5, SQL_C_DOUBLE, &planet.MassError, 0, &cbMassError);
		SQLBindCol(hstmt, 6, SQL_C_BIT, &planet.IsMassAMinimum, 0, &cbIsMassAMinimum);
		SQLBindCol(hstmt, 7, SQL_C_CHAR, &planet.MassSource, 51, &cbMassSource);
		SQLBindCol(hstmt, 8, SQL_C_DOUBLE, &planet.Radius, 0, &cbRadius);
		SQLBindCol(hstmt, 9, SQL_C_DOUBLE, &planet.RadiusError, 0, &cbRadiusError);
		SQLBindCol(hstmt, 10, SQL_C_CHAR, &planet.RadiusSource, 51, &cbRadiusSource);
		while (true)
		{
			retcode = SQLFetch(hstmt);
			if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
				break;
			planet.bMass = cbMass != SQL_NULL_DATA;
			planet.bMassSource = cbMassSource != SQL_NULL_DATA;
			planet.bMassError = cbMassError != SQL_NULL_DATA;
			planet.bRadius = cbRadius != SQL_NULL_DATA;
			planet.bRadiusSource = cbRadiusSource != SQL_NULL_DATA;
			planet.bRadiusError = cbRadiusError != SQL_NULL_DATA;
			if (CompareDates (&planet.LastUpdated, &sMostRecentUpdate) > 0)
				sMostRecentUpdate = planet.LastUpdated;

			sRawPlanets [sNumPlanets] = planet;
			++ sNumPlanets;
		}
	}
}


static int FindOrbitIndex (long orbitID)
{
	for (register i = 0; i < sNumOrbits; ++i)
		if (sRawOrbits[i].ID == orbitID)
			return	i;
	return	-1;
}

static int FindComponentIndex (long componentID)
{
	for (register i = 0; i < sNumComponents; ++i)
		if (sRawComponents[i].ID == componentID)
			return	i;
	return	-1;
}

static int FindPlanetIndex (long planetID)
{
	for (register i = 0; i < sNumPlanets; ++i)
		if (sRawPlanets[i].ID == planetID)
			return	i;
	return	-1;
}

// Danger Will Robinson -- recursive function ahead!
static void BuildOrbitPartNameAndLuminosity
	(char *name, double &luminosity, bool &isLuminosityValid, long ID)
{
	if ((ID % 100) < 10)	// it's a component
	{
		int i = FindComponentIndex (ID);
		if (i == -1)
			strcat (name, "<ERROR>");
		else
		{
			strcat (name, sRawComponents[i].Name);
			if (sDerivedComponents[i].VLuminosity == -1.0)
				isLuminosityValid = false;
			else
				luminosity += sDerivedComponents[i].VLuminosity;
		}
	}
	else if ((ID % 100) < 50)	// it's another orbit
	{
		int i = FindOrbitIndex (ID);
		if (i == -1)
			strcat (name, "<ERROR>");
		else
		{
			strcat (name, "(");
			BuildOrbitPartNameAndLuminosity
				(name, luminosity, isLuminosityValid, sRawOrbits[i].PrimaryID);
			strcat (name, "-");
			BuildOrbitPartNameAndLuminosity
				(name, luminosity, isLuminosityValid, sRawOrbits[i].SecondaryID);
			strcat (name, ")");
			if (!isLuminosityValid  &&  sRawOrbits[i].bHasMagnitude  &&
				sSystemPosition.bDistance)
			{
				double absVMag =
					sRawOrbits[i].mag.ApparentVMag + 5.0 -
					5.0 * log10 (sSystemPosition.Distance / LY_PER_PARSEC);

				luminosity = pow (2.512, 4.85 - absVMag);
				isLuminosityValid = true;
			}
		}
	}
	else	// it's a planet
	{
		int i = FindPlanetIndex (ID);
		if (i == -1)
			strcat (name, "<ERROR>");
		else
			strcat (name, sRawPlanets[i].Name);
		// Planets are not considered to contribute to the total luminosity
		// of a star system in any appreciable way.
	}
}

static void DisplayOrbits (int planetID = 0)
{
	long	primaryIDs[40];
	long	secondaryIDs[40];

	for (register i = 0; i < sNumOrbits; ++i)
	{
		double	primaryLuminosity = 0.0, secondaryLuminosity = 0.0;
		bool	primaryLumValid = true, secondaryLumValid = true;
		int		prevIdentical = 0;
		char	primaryName[60], secondaryName[60];
		double	actSeparation = -1.0, actPeriastron = -1.0,
				actApastron = -1.0;	// actual distances in A.U.s if known
		double	czDist = -1.0;	// comfort zone around both, in A.U.s
		bool	firstIsPlanet = (sRawOrbits[i].PrimaryID % 100) >= 51  &&
								(sRawOrbits[i].PrimaryID % 100) <= 69;
		bool	secondIsPlanet = (sRawOrbits[i].SecondaryID % 100) >= 51  &&
								(sRawOrbits[i].SecondaryID % 100) <= 69;

		if (planetID)
		{
			if (sRawOrbits[i].PrimaryID != planetID  &&
				sRawOrbits[i].SecondaryID != planetID)
					continue;
		}
		else
		{
			if ((firstIsPlanet || secondIsPlanet) != (planetID != 0))
				continue;
		}

		primaryIDs[i] = sRawOrbits[i].PrimaryID;
		secondaryIDs[i] = sRawOrbits[i].SecondaryID;
		for (register j = 0; j < i; ++j)
			if (primaryIDs[j] == primaryIDs[i]  &&
				secondaryIDs[j] == secondaryIDs[i])
					++prevIdentical;

		primaryName[0] = secondaryName[0] = '\0';
		BuildOrbitPartNameAndLuminosity (primaryName, primaryLuminosity,
			primaryLumValid, sRawOrbits[i].PrimaryID);
		BuildOrbitPartNameAndLuminosity (secondaryName, secondaryLuminosity,
			secondaryLumValid, sRawOrbits[i].SecondaryID);
		if (prevIdentical)
		{
			if (planetID)
				printf ("<BR>&mdash; OR &mdash;\n");
			else
				printf ("<P><B><U>Data for %s and %s's orbit, take %d:</U></B>\n",
					primaryName, secondaryName, prevIdentical + 1);
		}
		else
		{
			if (!planetID)
				printf ("<P><B><U>Data for %s and %s's orbit around one other:</U></B>\n",
					primaryName, secondaryName);
			if (sRawOrbits[i].bHasSpectrum)
			{
				if (sRawOrbits[i].spec.bSpectralClass)
				{
					printf ("<BR>");
					if (planetID)
						printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
					printf ("<B>Combined spectrum:</B> %s",
						sRawOrbits[i].spec.SpectralClass);
					if (sRawOrbits[i].spec.bLuminosityClass)
						printf (" %s", sRawOrbits[i].spec.LuminosityClass);
					printf ("\n");
				}
				else if (sRawOrbits[i].spec.bLuminosityClass)
				{
					printf ("<BR>");
					if (planetID)
						printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
					printf ("<B>Combined luminosity class:</B> %s\n",
						sRawOrbits[i].spec.LuminosityClass);
				}
			}

			double	combinedVLum = -1.0;
			if (primaryLumValid  &&  secondaryLumValid)
				combinedVLum = primaryLuminosity + secondaryLuminosity;
			else if (sRawOrbits[i].bHasMagnitude)
			{
				if (!planetID)
					printf ("<BR><B>Combined apparent visual magnitude:</B> %+.2f\n",
						sRawOrbits[i].mag.ApparentVMag);
				if (sSystemPosition.bDistance)
				{
					double absVMag =
						sRawOrbits[i].mag.ApparentVMag + 5.0 -
						5.0 * log10 (sSystemPosition.Distance / LY_PER_PARSEC);

					combinedVLum = pow (2.512, 4.85 - absVMag);
				}
			}
			if (!planetID  &&  combinedVLum > 0.0)
			{
				const double combinedAbsVMag = 4.85 - 2.5 * log10 (combinedVLum);

				printf ("<BR><B>Combined absolute visual magnitude:</B> %+.2f\n",
					combinedAbsVMag);
				printf ("<BR><B>Combined visual luminosity:</B> ");
				if (sSystemPosition.bDistanceError)
					printf ("%s", FormatWithSignificantDigits
						(combinedVLum, sSystemPosition.DistanceError));
				else if (combinedVLum < 0.1)
					printf ("%f", combinedVLum);
				else
					printf ("%g", combinedVLum);
				printf (" x Sol\n");
				czDist = sqrt (combinedVLum);
			}
		}

		if (sRawOrbits[i].bPeriod)
		{
			printf ("<BR>");
			if (planetID)
				printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
			printf ("<B>Period:</B> ");
			if (sRawOrbits[i].bPeriodError)
			{
				if (sRawOrbits[i].Period < 1.0/365.25)
					printf ("%s hours\n",
						FormatWithSignificantDigits
							(sRawOrbits[i].Period * 365.25 * 24.0, sRawOrbits[i].PeriodError));
				else if (sRawOrbits[i].Period < 1.0)
					printf ("%s days\n",
						FormatWithSignificantDigits
							(sRawOrbits[i].Period * 365.25, sRawOrbits[i].PeriodError));
				else
					printf ("%s years\n",
						FormatWithSignificantDigits
							(sRawOrbits[i].Period, sRawOrbits[i].PeriodError));
			}
			else
			{
				if (sRawOrbits[i].Period < 1.0/365.25)
					printf ("%f hours\n", sRawOrbits[i].Period * 365.25 * 24.0);
				else if (sRawOrbits[i].Period < 1.0)
					printf ("%g days\n", sRawOrbits[i].Period * 365.25);
				else
					printf ("%g years\n", sRawOrbits[i].Period);
			}
		}
		if (sRawOrbits[i].bPeriodError)
		{
			printf ("<BR>");
			if (planetID)
				printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
			printf ("<B>Standard Error in period:</B> ");
			if (sRawOrbits[i].PeriodError < 0.00001)
				printf ("%f", sRawOrbits[i].PeriodError * 100.0);
			else
				printf ("%g", sRawOrbits[i].PeriodError * 100.0);
			printf ("%%\n");
		}
		if (sRawOrbits[i].bSemimajorAxis)
		{
			if (sRawOrbits[i].IsAxisAU)
				actSeparation = sRawOrbits[i].SemimajorAxis;
			else if (sSystemPosition.bDistance)
				actSeparation = sRawOrbits[i].SemimajorAxis *
					sSystemPosition.Distance / LY_PER_PARSEC;
			printf ("<BR>");
			if (planetID)
				printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
			if (sRawOrbits[i].IsAxisJustGap)
				printf ("<B>Observed Separation:</B> ");
			else
				printf ("<B>Semimajor Axis:</B> ");
			if (actSeparation == -1.0)
			{
				if (sRawOrbits[i].SemimajorAxis < 0.001)
					printf ("%f", sRawOrbits[i].SemimajorAxis);
				else
					printf ("%g", sRawOrbits[i].SemimajorAxis);
				printf (" arcseconds\n");
			}
			else if (!sRawOrbits[i].IsAxisAU  &&  sSystemPosition.bDistanceError)
				printf ("%s A.U.s\n", FormatWithSignificantDigits
					(actSeparation, sSystemPosition.DistanceError));
			else if (actSeparation < 0.001)
				printf ("%f A.U.s\n", actSeparation);
			else
				printf ("%g A.U.s\n", actSeparation);
			if (!sRawOrbits[i].IsAxisJustGap  &&
				sRawOrbits[i].bEccentricity)
			{
				printf ("<BR>");
				if (planetID)
					printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				printf ("<B>Eccentricity:</B> %g\n", sRawOrbits[i].Eccentricity);
				printf ("<BR>");
				if (planetID)
					printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				printf ("<B>Periastron distance:</B> ");
				if (actSeparation != -1.0)
				{
					actPeriastron = actSeparation * (1.0 - sRawOrbits[i].Eccentricity);
					if (!sRawOrbits[i].IsAxisAU  &&  sSystemPosition.bDistanceError)
						printf ("%s A.U.s\n", FormatWithSignificantDigits
							(actPeriastron, sSystemPosition.DistanceError));
					else if (actPeriastron < 0.001)
						printf ("%f A.U.s\n", actPeriastron);
					else
						printf ("%g A.U.s\n", actPeriastron);
				}
				else
					printf ("%f arcseconds\n", sRawOrbits[i].SemimajorAxis *
						(1.0 - sRawOrbits[i].Eccentricity));
				printf ("<BR>");
				if (planetID)
					printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				printf ("<B>Apastron distance:</B> ");
				if (actSeparation != -1.0)
				{
					actApastron = actSeparation * (1.0 + sRawOrbits[i].Eccentricity);
					if (!sRawOrbits[i].IsAxisAU  &&  sSystemPosition.bDistanceError)
						printf ("%s A.U.s\n", FormatWithSignificantDigits
							(actApastron, sSystemPosition.DistanceError));
					else if (actApastron < 0.001)
						printf ("%f A.U.s\n", actApastron);
					else
						printf ("%g A.U.s\n", actApastron);
				}
				else
					printf ("%f arcseconds\n", sRawOrbits[i].SemimajorAxis *
						(1.0 + sRawOrbits[i].Eccentricity));
			}
			if (sRawOrbits[i].bPeriastronEpoch)
			{
				printf ("<BR>");
				if (planetID)
					printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				if (sRawOrbits[i].IsAxisJustGap)
					printf ("<B>Year of separation observation:</B> ");
				else
					printf ("<B>Year in which periastron occurs:</B> ");
				printf ("%g\n", sRawOrbits[i].PeriastronEpoch);
			}
			if (sRawOrbits[i].bSource)
			{
				printf ("<BR>");
				if (planetID)
					printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				printf ("<B>Source for orbit data:</B> %s\n",
					CatAbbrev2LongName (sRawOrbits[i].Source));
			}

			if (!planetID)
			{
				if (czDist != -1.0  &&  actSeparation != -1.0  &&
					czDist >= 2.0 * actSeparation)
				{
					printf ("<BR><B>Comfort Zone (visual) around both:</B> ");
					if (sSystemPosition.bDistanceError)
						printf ("%s", FormatWithSignificantDigits
							(czDist, sSystemPosition.DistanceError));
					else if (czDist < 0.001)
						printf ("%f", czDist);
					else
						printf ("%g", czDist);
					printf (" A.U.s\n");
				}

				if (secondaryLumValid  &&  secondaryLuminosity > 0.0)
				{
					double secondaryAbsMag = 4.85 - log10(secondaryLuminosity) * 2.5;
					if (actSeparation != -1.0)
					{
						printf ("<BR><B>As seen from %s:</B>\n", primaryName);
						if (actPeriastron != -1.0)
						{
							printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;At periastron, "
								"%s would appear as magnitude %+.2f\n",
								secondaryName,
								secondaryAbsMag - 5.0 + 5.0 *
									log10(actPeriastron / 206265.0));
							printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;At apastron, "
								"%s would appear as magnitude %+.2f\n",
								secondaryName,
								secondaryAbsMag - 5.0 + 5.0 *
									log10(actApastron / 206265.0));
						}
						else
							printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%s would appear as magnitude %+.2f\n",
								secondaryName,
								secondaryAbsMag - 5.0 + 5.0 *
									log10(actSeparation / 206265.0));
					}
				}

				if (primaryLumValid  &&  primaryLuminosity > 0.0)
				{
					double primaryAbsMag = 4.85 - log10(primaryLuminosity) * 2.5;
					if (actSeparation != -1.0)
					{
						printf ("<BR><B>As seen from %s:</B>\n", secondaryName);
						if (actPeriastron != -1.0)
						{
							printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;At periastron, "
								"%s would appear as magnitude %+.2f\n",
								primaryName,
								primaryAbsMag - 5.0 + 5.0 *
									log10(actPeriastron / 206265.0));
							printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;At apastron, "
								"%s would appear as magnitude %+.2f\n",
								primaryName,
								primaryAbsMag - 5.0 + 5.0 *
									log10(actApastron / 206265.0));
						}
						else
							printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;%s would appear as magnitude %+.2f\n",
								primaryName,
								primaryAbsMag - 5.0 + 5.0 *
									log10(actSeparation / 206265.0));
					}
				}
			}
		}
		else
		{
			if (sRawOrbits[i].bEccentricity)
			{
				printf ("<BR>");
				if (planetID)
					printf ("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
				printf ("<B>Eccentricity:</B> %g\n", sRawOrbits[i].Eccentricity);
			}
			if (!planetID  &&  czDist != -1.0)
			{
				printf ("<BR><B>Comfort Zone (visual) around both:</B> ");
				if (sSystemPosition.bDistanceError)
					printf ("%s", FormatWithSignificantDigits
						(czDist, sSystemPosition.DistanceError));
				else if (czDist < 0.1)
					printf ("%f", czDist);
				else
					printf ("%g", czDist);
				printf (" A.U.s\n");
			}
		}
	}
}


// The following variable star type abbreviations and their long
// descriptions come verbatim from the Hipparcos Input Catalog.
static const char *GetVariableTypeName
	(const char *pVariableType, const char *&pVariableClass)
{
	const struct
	{
		const char *Abbrev;
		const char *Name;
		const char *Class;
	} varTypes[] =
	{
		{ "ACV", "Alpha(2) Canum Venaticorum", "rotating variable" },
		{ "ACY", "Alpha Cygni", "pulsating variable" },
		{ "BCE", "Beta Cephei", "pulsating variable" },
		{ "BY", "BY Draconis", "rotating variable" },
		{ "CEP", "Cepheid", "pulsating variable" },
		{ "CST", "constant star", "formally believed variable by some observer" },
		{ "CW", "W Virginis", "pulsating variable" },
		{ "CWA", "W Virginis (period > 8 days)", "pulsating variable" },
		{ "CWB", "W Virginis (period < 8 days)", "pulsating variable" },
		{ "DCE", "delta Cephei", "pulsating variable" },
		{ "DSC", "delta Scuti", "pulsating variable" },
		{ "E", "(E+, E/ ..)", "eclipsing binary" },
		{ "EA", "Algol", "eclipsing binary" },
		{ "EB", "beta Lyrae", "eclipsing binary" },
		{ "ELL", "rotating ellipsoidal", "rotating variable" },
		{ "EW", "W Ursae Majoris", "eclipsing binary" },
		{ "FKC", "FK Comae Berenices ", "rotating variable" },
		{ "GCA", "gamma Cassiopeiae", "eruptive variable" },
		{ "I", "irregular", " eruptive variable" },
		{ "IN", "irregular", "eruptive variable" },
		{ "IS", "irregular", " eruptive variable" },
		{ "L", "slow irregular", "pulsating variable" },
		{ "M", "Mira Ceti", "pulsating variable" },
		{ "NA", "fast nova", "cataclysmic variable" },
		{ "NB", "slow nova", "cataclysmic variable" },
		{ "NC", "slow nova", "cataclysmic variable" },
		{ "NL", "nova-like", "cataclysmic variable" },
		{ "NR", "recurrent nova", "cataclysmic variable" },
		{ "PVT", "PV Telescopii", "pulsating variable" },
		{ "RCB", "R Coronae Borealis", "eruptive variable" },
		{ "RR", "RR Lyrae", "pulsating variable" },
		{ "RS", "RS Canum Venaticorum", "eclipsing binary" },
		{ "RV", "RV Tauri", "pulsating variable" },
		{ "SDO", "S Doradus", "eruptive variable" },
		{ "SR", "semi-regular", "pulsating variable" },
		{ "SXA", "SX Arietis", "rotating variable" },
		{ "SXP", "SX Phoenicis", "pulsating variable" },
		{ "UGS", "SS Cygni", "cataclysmicvariable" },
		{ "UV", "UV Ceti <a href=\"/flare.html\">flare star</a>", "eruptive variable" },
		{ "WR", "Wolf-Rayet", "eruptive variable" },
		{ "XNG", "X-ray nova-like system", "X-ray binary" },
		{ "XP", "X-ray pulsar", "X-ray binary" },
		{ "ZAN", "Z Andromedae", "cataclysmic variable" },
		{ "ZZA", "Hydrogen ZZ Ceti", "pulsating variable" }
	};
	const numVarTypes = sizeof varTypes / sizeof varTypes[0];

	for (register i = 0; i < numVarTypes; ++i)
		if (!strcmp (pVariableType, varTypes[i].Abbrev))
		{
			pVariableClass = varTypes[i].Class;
			return	varTypes[i].Name;
		}
	pVariableClass = NULL;
	return	NULL;
}

static bool DoesThisOrbitInvolveComponent (int orbitIndex, int componentID)
{
	RawOrbit &orbit = sRawOrbits [orbitIndex];
	if (orbit.PrimaryID == componentID  ||
		orbit.SecondaryID == componentID)
			return true;
	for (register i = 0; i < sNumOrbits; ++i)
		if (sRawOrbits[i].PrimaryID == orbit.ID  ||
			sRawOrbits[i].SecondaryID == orbit.ID)
		{
			if (sRawOrbits[i].PrimaryID == componentID  ||
				sRawOrbits[i].SecondaryID == componentID  ||
				DoesThisOrbitInvolveComponent (i, componentID))
					return true;
		}
	return false;
}

static void GetPlanetsInvolvingComponent
	(int componentID, int &numPlanets, int planetIDs[])
{
	numPlanets = 0;

	for (register orbitIndex = 0; orbitIndex < sNumOrbits; ++orbitIndex)
	{
		RawOrbit &orbit = sRawOrbits [orbitIndex];
		if ((orbit.PrimaryID % 100) > 50  ||  (orbit.SecondaryID % 100) > 50)
		{
			// This orbit involves a planet.  Follow the chain of orbits
			// backward to see if one of the parents involves componentID:
			if (DoesThisOrbitInvolveComponent (orbitIndex, componentID))
			{
				if ((orbit.PrimaryID % 100) > 50)
				{
					for (register i = 0; i < numPlanets; ++i)
						if (planetIDs[i] == orbit.PrimaryID)
							break;
					if (i >= numPlanets)
						planetIDs[numPlanets++] = orbit.PrimaryID;
				}
				if ((orbit.SecondaryID % 100) > 50)
				{
					for (register i = 0; i < numPlanets; ++i)
						if (planetIDs[i] == orbit.SecondaryID)
							break;
					if (i >= numPlanets)
						planetIDs[numPlanets++] = orbit.SecondaryID;
				}
			}
		}
	}
}

static void FetchAndDisplayComponents(HSTMT hstmt)
{
	char	buffer[512];
	RETCODE	retcode;

	for (int index = 0; index < sNumComponents; ++index)
	{
		if (sNumComponents > 1)
			printf ("<P><BR><B><U>Component %s:</U></B>\n",
				sRawComponents[index].Name);
		else
			printf ("<BR>\n");

		if (sRawComponents[index].IsCloseBinary)
			printf ("<BR>NOTE: This star is actually a close-orbiting "
					"binary pair.  The data below are for both sub-components "
					"of the pair combined.\n");

		SQLFreeStmt (hstmt, SQL_CLOSE);
		sprintf (buffer, "SELECT Name FROM ProperNames "
			"WHERE OwnerID = %ld", sRawComponents[index].ID);
		retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
		if (retcode == SQL_SUCCESS)
		{
			SDWORD	cbName;
			char	name[51];
			bool	printedOne = false;

			SQLBindCol(hstmt, 1, SQL_C_CHAR, name, 51, &cbName);
			while (true)
			{
				retcode = SQLFetch(hstmt);
				if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
					break;
				if (printedOne)
					printf (", ");
				else
				{
					printf ("<BR><B>Proper names:</B> ");
					printedOne = true;
				}
				printf ("%s", name);
			}
			if (printedOne)
				printf ("\n");
		}

		SQLFreeStmt (hstmt, SQL_CLOSE);
		sprintf (buffer, "SELECT CatAbbrev, Value FROM CatalogNumbers "
			"WHERE OwnerID = %ld", sRawComponents[index].ID);
		retcode = SQLExecDirect (hstmt, (unsigned char *) buffer, SQL_NTS);
		if (retcode == SQL_SUCCESS)
		{
			char	catAbbrev[11], value[51];
			SDWORD	cbCatAbbrev, cbValue;
			bool	printedOne = false;

			SQLBindCol(hstmt, 1, SQL_C_CHAR, catAbbrev, 11, &cbCatAbbrev);
			SQLBindCol(hstmt, 2, SQL_C_CHAR, value, 51, &cbValue);
			while (true)
			{
				retcode = SQLFetch(hstmt);
				if (retcode != SQL_SUCCESS  &&  retcode != SQL_SUCCESS_WITH_INFO)
					break;
				if (!printedOne)
				{
					printf ("<BR><B>Catalog numbers:</B>\n<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
					printedOne = true;
				}
				else
					printf (", ");
				CatAbbrev2LongNamePlusAbbrev (catAbbrev, buffer);
				printf ("%s %s", buffer, value);
			}
			// if (printedOne)
			//	printf ("<BR>\n");
		}

		if (sComponentPosition[index].bRefinedRA  ||
			sComponentPosition[index].bRefinedDec  ||
			sComponentPosition[index].bRefinedDist)
		{
			DisplayRADecDist (&sComponentPosition[index], true);
			DisplayCoords (&sComponentPosition[index]);
		}

		if (sComponentVelocity[index].bRefined)
		{
			double	u, v, w;

			DisplayPMPARV (&sComponentVelocity[index]);
			if (RDDPAR2UVW (&sComponentPosition[index],
							&sComponentVelocity[index], u, v, w, true))
			{
				printf ("<BR><B>Galactic (U,V,W) velocity components in km/s:</B> ");
				Display3DistanceDependentValues (u, v, w, &sComponentPosition[index]);
				printf ("\n");
			}
		}

		if (sRawComponents[index].bHasSpectrum)
		{
			if (sRawComponents[index].spec.bSpectralClass)
				printf ("<BR><B>Spectral class:</B> %s\n",
					sRawComponents[index].spec.SpectralClass);
			if (sRawComponents[index].spec.bLuminosityClass)
				printf ("<BR><B>Luminosity Class:</B> %s\n",
					sRawComponents[index].spec.LuminosityClass);
		}
		if (sRawComponents[index].bHasMagnitude  &&
			sRawComponents[index].mag.bApparentVMag)
		{
			printf ("<BR><B>Apparent visual magnitude:</B> %+.2f",
				sRawComponents[index].mag.ApparentVMag);
			if (sRawComponents[index].mag.bVMagIncrease  &&
				sRawComponents[index].mag.VMagIncrease > 0.0)
					printf (" (increasing to %+.2f)",
						sRawComponents[index].mag.ApparentVMag -
						sRawComponents[index].mag.VMagIncrease);
			printf ("\n");
		}
		if (sDerivedComponents[index].AbsVMag != -9999.0)
		{
			printf ("<BR><B>Absolute visual magnitude:</B> %+.2f",
				sDerivedComponents[index].AbsVMag);
			if (sDerivedComponents[index].IncreasedAbsVMag < sDerivedComponents[index].AbsVMag)
				printf (" (increasing to %+.2f)", sDerivedComponents[index].IncreasedAbsVMag);
			printf ("\n");
		}
		if (sDerivedComponents[index].VLuminosity != -1.0)
		{
			printf ("<BR><B>Visual luminosity:</B> ");
			if (sComponentPosition[index].bDistanceError)
				printf ("%s", FormatWithSignificantDigits
					(sDerivedComponents[index].VLuminosity,
					sComponentPosition[index].DistanceError));
			else if (sDerivedComponents[index].VLuminosity < 0.1)
				printf ("%f", sDerivedComponents[index].VLuminosity);
			else
				printf ("%g", sDerivedComponents[index].VLuminosity);
			printf (" x Sol");

			if (sDerivedComponents[index].IncreasedVLuminosity >
				sDerivedComponents[index].VLuminosity)
			{
				printf (" (increasing to  ");
				if (sComponentPosition[index].bDistanceError)
					printf ("%s", FormatWithSignificantDigits
						(sDerivedComponents[index].IncreasedVLuminosity,
						sComponentPosition[index].DistanceError));
				else if (sDerivedComponents[index].IncreasedVLuminosity < 0.1)
					printf ("%f", sDerivedComponents[index].IncreasedVLuminosity);
				else
					printf ("%g", sDerivedComponents[index].IncreasedVLuminosity);
				printf (" x Sol)");
			}
			printf ("\n");
		}
		if (sRawComponents[index].bHasMagnitude)
		{
			RawMagnitude &mag = sRawComponents[index].mag;
			if (mag.bVariableType)
			{
				const char *pVariableTypeName, *pVariableClass;
				printf ("<BR><B>Variable type:</B> ");
				pVariableTypeName = GetVariableTypeName (mag.VariableType, pVariableClass);
				if (pVariableTypeName)
					printf ("%s (%s)\n", pVariableTypeName, pVariableClass);
				else
					puts (mag.VariableType);
			}
			if (mag.bVariablePeriod)
			{
				printf ("<BR><B>Period of variability:</B> ");
				if (mag.VariablePeriod >= 1.0)
					printf ("%g days\n", mag.VariablePeriod);
				else if (mag.VariablePeriod >= 1.0/24.0)
					printf ("%g hours\n", mag.VariablePeriod * 24.0);
				else if (mag.VariablePeriod >= 1.0/1440.0)
					printf ("%g minutes\n", mag.VariablePeriod * 1440.0);
				else
					printf ("%g seconds\n", mag.VariablePeriod * 86400.0);
				printf ("\n");
			}
			bool bPrintedColorIndex = false;
			if (mag.bBminusV)
			{
				printf ("<BR><B>Color indices:</B> B-V= %+.2f", mag.BminusV);
				bPrintedColorIndex = true;
			}
			if (mag.bUminusB)
			{
				if (bPrintedColorIndex)
					printf (", ");
				else
				{
					printf ("<BR><B>Color indices:</B> ");
					bPrintedColorIndex = true;
				}
				printf ("U-B= %+.2f", mag.UminusB);
			}
			if (mag.bRminusI)
			{
				if (bPrintedColorIndex)
					printf (", ");
				else
				{
					printf ("<BR><B>Color indices:</B> ");
					bPrintedColorIndex = true;
				}
				printf ("R-I= %+.2f", mag.RminusI);
			}
			if (bPrintedColorIndex)
				printf ("\n");
		}
		if (sRawComponents[index].bMass)
		{
			printf ("<BR><B>Mass:</B> %g x Sol\n", sRawComponents[index].Mass);
			if (sRawComponents[index].bMassSource)
				printf ("<BR><B>Source for mass:</B> %s\n",
					CatAbbrev2LongName (sRawComponents[index].MassSource));
		}
		double	radius = 0.0;	// in solar radii
		if (sRawComponents[index].bDiameter)
			if (!sRawComponents[index].IsDiameterArcsec)
			{
				radius = sRawComponents[index].Diameter;
				printf ("<BR><B>Diameter:</B> ");
				if (radius < 0.001)
					printf ("%g kilometers\n", 1391980.0 * radius);
				else
					printf ("%g x Sol\n", radius);
			}
			else if (sComponentPosition[index].bDistance)
			{
				radius = sRawComponents[index].Diameter * 206265.0 / LY_PER_PARSEC
					/ 3600.0 / 0.533111 * sComponentPosition[index].Distance;
				printf ("<BR><B>Diameter:</B> ");
				if (sComponentPosition[index].bDistanceError)
				{
					if (radius < 0.001)
						printf ("%s kilometers\n", FormatWithSignificantDigits
							(1391980.0 * radius, sComponentPosition[index].DistanceError));
					else
						printf ("%s x Sol\n", FormatWithSignificantDigits
							(radius, sComponentPosition[index].DistanceError));
				}
				else if (radius < 0.001)
					printf ("%g kilometers\n", 1391980.0 * radius);
				else
					printf ("%g x Sol\n", radius);
			}
		if (sRawComponents[index].bDiameterSource)
			printf ("<BR><B>Source for diameter:</B> %s\n",
				CatAbbrev2LongName (sRawComponents[index].DiameterSource));
		if (sDerivedComponents[index].ComfortZoneAUs != -1.0)
		{
			double	czDist = sDerivedComponents[index].ComfortZoneAUs;

			printf ("<BR><B>Comfort Zone (visual):</B> ");
			if (sComponentPosition[index].bDistanceError)
				printf ("%s", FormatWithSignificantDigits (czDist,
					sComponentPosition[index].DistanceError));
			else if (czDist < 0.1)
				printf ("%f", czDist);
			else
				printf ("%g", czDist);
			printf (" A.U.s\n");

			if (sRawComponents[index].bMass  &&
				sRawComponents[index].Mass)
			{
				double	czPeriod = sqrt (pow (czDist, 3) / sRawComponents[index].Mass);

				printf ("<BR><B>Orbital period in CZ:</B> ");
				if (czPeriod < 1.0/365.25)
					printf ("%g hours\n", czPeriod * 365.25 * 24.0);
				else if (czPeriod < 1.0)
					printf ("%g days\n", czPeriod * 365.25);
				else
					printf ("%g years\n", czPeriod);

				double tidalIndex = sRawComponents[index].Mass / pow (czDist, 3);
				printf ("<BR><B>Tidal index in CZ:</B> ");
				if (tidalIndex < 0.1)
					printf ("%f\n", tidalIndex);
				else
					printf ("%g\n", tidalIndex);
			}
			if (radius)
				printf ("<BR><B>Angular size of star in sky in CZ:</B> %f degrees\n",
					radius / czDist * 0.533111);
		}

		// Show all the planets/brown dwarfs for this component, including
		// any that are orbiting other planets:
		int nPlanetsInvolvingThisComponent = 0;
		int planetIDsInvolvingThisComponent [19];
		GetPlanetsInvolvingComponent (sRawComponents[index].ID,
			nPlanetsInvolvingThisComponent,
			planetIDsInvolvingThisComponent);
		if (nPlanetsInvolvingThisComponent)
			printf ("<BR><B>Detected companions:</B> %d\n",
					nPlanetsInvolvingThisComponent);
		for (register planetIDIndex = 0;
			 planetIDIndex < nPlanetsInvolvingThisComponent;
			 ++planetIDIndex)
		{
			for (register int i = 0; i < sNumPlanets; ++i)
				if (sRawPlanets[i].ID == planetIDsInvolvingThisComponent [planetIDIndex])
					break;
			RawPlanet &planet = sRawPlanets [i];
			printf ("<BR><B><U>Companion %s:</U></B>\n", planet.Name);
			if (planet.bMass)
			{
				printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<B>Mass:</B> ");
				if (planet.Mass < 0.001)
					printf ("%f", planet.Mass);
				else
					printf ("%g", planet.Mass);
				printf (" x Jupiter");
				if (planet.IsMassAMinimum)
					printf (", minimum");
				printf ("\n");
				if (planet.bMassError)
					printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
						"<B>Standard error in mass:</B> %g%%\n",
						planet.MassError * 100.0);
				if (planet.bMassSource)
					printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
						"<B>Source for mass:</B> %s\n",
						CatAbbrev2LongName (planet.MassSource));
			}
			if (planet.bRadius)
			{
				printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<B>Diameter:</B> ");
				if (planet.Radius < 0.001)
					printf ("%f", planet.Radius);
				else
					printf ("%g", planet.Radius);
				printf (" x Jupiter\n");
				if (planet.bRadiusError)
					printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
						"<B>Standard error in diameter:</B> %g%%\n",
						planet.RadiusError * 100.0);
				if (planet.bRadiusSource)
					printf ("<BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
						"<B>Source for diameter:</B> %s\n",
						CatAbbrev2LongName (planet.RadiusSource));
			}
			DisplayOrbits (planet.ID);
		}
	}
}

static const char *MonthName (int month)
{
	const char *monthNames[] =
	{
		"January",
		"February",
		"March",
		"April",
		"May",
		"June",
		"July",
		"August",
		"September",
		"October",
		"November",
		"December"
	};

	if (month < 1 || month > 12)
		return	"";
	return	monthNames[month-1];
}

bool display_id (HDBC hdbc, HSTMT hstmt, long id)
{
	printf ("Content-type: text/html%c%c", 10, 10);
	puts ("<HTML>");
	puts ("<HEAD>");
	puts ("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=windows-1252\">");
	puts ("<META NAME=\"viewport\" CONTENT=\"width=device-width, initial-scale=1\">");
	puts ("<TITLE>ISDB search results</TITLE>");
	puts ("</HEAD>");
	puts ("<BODY>");
	sMostRecentUpdate.year = 1900;
	sMostRecentUpdate.month = 1;
	sMostRecentUpdate.day = 1;

	FetchCatalogNames (hstmt);
	FetchRawComponents (hdbc, hstmt, id);
	FetchSystemAndComponentRawPositions (hstmt, id);
	ComputeDerivedComponents();
	FetchSystemAndComponentRawVelocities (hstmt, id);
	FetchAndDisplaySystemNames (hstmt, id);
	FetchAndDisplaySystemData (hstmt, id);
	DisplayRADecDist (&sSystemPosition, false);
	DisplayCoords (&sSystemPosition);
	DisplaySystemVelocity();
	puts ("<P><A HREF=\"/fields.html\">What do all these fields mean?</A><BR>");
	FetchRawOrbits (hdbc, hstmt, id);
	FetchRawPlanets (hdbc, hstmt, id);
	DisplayOrbits();
	FetchAndDisplayComponents (hstmt);
	DisposeCatalogNames();

	puts ("<FORM ACTION=\"/Scripts/find_neighbors.exe\" METHOD=\"GET\">");
	printf ("<INPUT TYPE=\"Hidden\" NAME=\"ID\" VALUE=\"%d\">\n", id);
	puts ("<INPUT TYPE=\"Submit\" VALUE=\"Find all stars within\">");
	puts ("<INPUT TYPE=\"text\" NAME=\"ly\" VALUE=\"15\" SIZE=3> light-years");
	puts ("</FORM>\n");

	puts ("<FORM ACTION=\"/Scripts/find_neighbors.exe\" METHOD=\"GET\">");
	printf ("<INPUT TYPE=\"Hidden\" NAME=\"ID\" VALUE=\"%d\">\n", id);
	puts ("<INPUT TYPE=\"Submit\" VALUE=\"Find all stars at least\">");
	puts ("<INPUT TYPE=\"text\" NAME=\"minLY\" VALUE=\"10\" SIZE=3> but not more than");
	puts ("<INPUT TYPE=\"text\" NAME=\"ly\" VALUE=\"15\" SIZE=3> light-years away");
	puts ("</FORM>\n");

	printf ("Data for this star system were most recently updated on %d-%s-%d.\n",
		sMostRecentUpdate.day, MonthName (sMostRecentUpdate.month), sMostRecentUpdate.year);
	puts ("</BODY>\n</HTML>");
	return	true;
}
